/**
 * 
 */
package softwarepulse.gui;

import java.awt.Color;
import java.awt.Font;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.EventListener;
import javax.swing.JButton;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JPasswordField;
import javax.swing.JSeparator;
import javax.swing.JTextField;
import javax.swing.SwingConstants;
import javax.swing.event.EventListenerList;

/**
 * Provides a login panel for user to enter user id and password
 * 
 * @author jmcneil
 * (c) copyright Software Pulse 2016
 *
 */
public class Login implements ActionListener {

	JPanel panelLogin;
	EventListenerList listeners = new EventListenerList();
	
	JLabel lblUser;
	JLabel lblPswd;
	JLabel lblError;	
	JButton btnOK;
	JButton btnCancel;
	JTextField txtUser;
	JPasswordField pswd;
	
	/**
	 * Default constructor 
	 */
	public Login() {
		super();
		intialise();
	}
	
	
	private void intialise() {
		panelLogin = new JPanel();
		Font fontLabel = new Font("Calibri", Font.PLAIN, 12);
		
		lblUser = new JLabel("Username: ");
		lblPswd = new JLabel("Password: ");
		lblError = new JLabel("Username or password is not valid.");
		lblError.setForeground(panelLogin.getBackground());
		
		lblUser.setFont(fontLabel);
		lblPswd.setFont(fontLabel);
		
		btnOK = new JButton("OK");
		btnCancel = new JButton("Cancel");
		
		btnOK.setFont(fontLabel);
		btnCancel.setFont(fontLabel);
		
		txtUser = new JTextField();
		pswd = new JPasswordField();
		
		GridBagLayout gbLayout = new GridBagLayout();
		GridBagConstraints gbc = new GridBagConstraints();
		
		panelLogin.setLayout(gbLayout);
	    
	    gbc.fill=GridBagConstraints.NONE;
		gbc.gridx = 1;
	    gbc.gridy = 5;
		gbc.insets = new Insets(6,10,17,10);
	    panelLogin.add(btnOK, gbc);
	    
		gbc.gridx = 2;
	    gbc.gridy = 5;
		gbc.insets = new Insets(6,10,17,20);
		panelLogin.add(btnCancel, gbc);
	    
		
	    gbc.fill=GridBagConstraints.NONE;
	    gbc.gridx = 0;
	    gbc.gridy = 2;
		gbc.insets = new Insets(6,20,0,10);
	    panelLogin.add(lblUser, gbc);
	    
		gbc.gridx = 0;
	    gbc.gridy = 3;
		gbc.insets = new Insets(6,20,0,10);
	    panelLogin.add(lblPswd, gbc);

	    gbc.fill=GridBagConstraints.HORIZONTAL;
		gbc.gridx = 1;
	    gbc.gridy = 2;
		gbc.insets = new Insets(6,10,0,20);
	    gbc.gridwidth = 2;
		panelLogin.add(txtUser, gbc);
	    
	    gbc.fill=GridBagConstraints.HORIZONTAL;
		gbc.gridx = 1;
	    gbc.gridy = 3;
		gbc.insets = new Insets(6,10,0,20);
	    gbc.gridwidth = 2;
		panelLogin.add(pswd, gbc);
	    

		gbc.fill=GridBagConstraints.HORIZONTAL;
		gbc.gridx = 0;
	    gbc.gridy = 4;
	    gbc.gridwidth = 4;
		gbc.insets = new Insets(17,1,10,1);
	    JSeparator sep = new JSeparator(SwingConstants.HORIZONTAL);
	    sep.setForeground(Color.DARK_GRAY);
	    panelLogin.add(sep, gbc);
	   
	    gbc.fill=GridBagConstraints.NONE;
		gbc.gridx = 0;
	    gbc.gridy = 1;
	    gbc.gridwidth = 4;
		gbc.insets = new Insets(0,0,0,0);
		lblError.setFont(fontLabel);
		
	    panelLogin.add(lblError, gbc);
	    
		gbc.fill=GridBagConstraints.HORIZONTAL;
		gbc.insets = new Insets(17,20,10,20);
		gbc.gridx = 0;
	    gbc.gridy = 0;
	    gbc.gridwidth = 3;
	    JLabel labelHeader = new JLabel("Please enter your username and password");
	    labelHeader.setFont(fontLabel);
	    panelLogin.add(labelHeader, gbc);
	    
	    btnOK.addActionListener(this);
		btnCancel.addActionListener(this);
		
	}
	
	
	/**
	 * @return the panelLogin
	 */
	public JPanel getPanelLogin() {
		return panelLogin;
	}
	
		

	/**
	 * @return the txtUser
	 */
	public JTextField getTxtUser() {
		return txtUser;
	}


	/**
	 * @return the pswd
	 */
	public JPasswordField getPswd() {
		return pswd;
	}


	public void addActionListener(ActionListener arg0) {
		listeners.add(ActionListener.class, arg0);
	}


	protected void fireActionPerformed(String id) {
		 EventListener[] l = listeners.getListeners(ActionListener.class);
		    ActionEvent e = null;
		    for (int i = 0; i < l.length; i++) {
		        if (e == null) {
		            e = new ActionEvent(this, ActionEvent.ACTION_PERFORMED, id);
		        }
		        ((ActionListener) l[i]).actionPerformed(e);	       
		    }
	}
	
	@Override
	 public void actionPerformed(ActionEvent event) { 
		if(event.getActionCommand().equals("OK")) {
			// check we have values in both user and password fields
			char[] charPswd = pswd.getPassword();
			
			if( txtUser.getText().length()> 0 && charPswd.length>0 ) {
				fireActionPerformed(event.getActionCommand());			
			} else {
				invalidEntry();
			}
		} else if(event.getActionCommand().equals("Cancel")) {
			// Cancel login
			fireActionPerformed(event.getActionCommand());
		} else if(event.paramString().split("cmd=")[1].split(",")[0].equals("NOTAUTH")) {
			invalidEntry();
		}
	 }


	private void invalidEntry() {
		lblError.setText("Username or password is not valid.");
		lblError.setForeground(Color.RED);
		pswd.setText(null);
		panelLogin.validate();
		panelLogin.repaint();
	}
	
}
