/**
 * 
 */
package softwarepulse.util;

import java.io.ObjectInputStream.GetField;
import java.security.MessageDigest;
import java.security.NoSuchAlgorithmException;
import java.security.SecureRandom;
import java.sql.ResultSet;
import java.sql.SQLException;

import org.apache.log4j.Logger;

import softwarepulse.db.DBManager;

/**
 * @author jmcneil
 * (c) copyright Software Pulse 2016
 *
 */
public class Authenticate {

	
	static DBManager userDB = new DBManager();
	static Logger log = Logger.getLogger(Authenticate.class);

	public static boolean isValid(String user, String pswd) throws ClassNotFoundException, SQLException, NoSuchAlgorithmException {
		log.debug("isValid() with: " + user + " and " + pswd);
		// Lookup user credentials in database
		ResultSet rs = userDB.getUser(user);
		if( rs.next()) {
			// user name found
			log.debug("User : " + user + " found now check password");
			String resPswd = rs.getString("pswd");
			String salt = rs.getString("salt");
			
			// compare passwords
			return comparePswd(pswd, resPswd, salt);
		}  
		return false;
	}
	
	public static boolean createUser(String user, String pswd) throws ClassNotFoundException, SQLException, NoSuchAlgorithmException {
		log.debug("createUser() with: " + user + " and " + pswd);
		// Check for duplicates
		ResultSet rs5 = userDB.getUser(user);
		if( rs5.next()) {
			// user name already in use
			log.debug("name already in use: " + rs5.getString("user"));
			return false;
		}  
					
		// Salt pswd
		byte[] bytesPswd = pswd.getBytes();
		int salt = generateSalt();
		pswd = salt + pswd;
		bytesPswd = pswd.getBytes();
		
		// hash pswd
		byte[] hashPswd = hashPswd(bytesPswd);
		
		// insert user into db
		userDB.addUser(user, new String(hashPswd), Integer.toString(salt));
		return true;
	}

	private static boolean comparePswd(String pswd, String resPswd, String salt) throws NoSuchAlgorithmException {
		// Prepend salt to pswd
		byte[] pswdToValidate = (salt + pswd).getBytes();
		
		// hash pswd to validate
		byte[] hash = hashPswd(pswdToValidate);

		// check to see if passwords match
		byte[] origPswd = resPswd.getBytes();
		log.debug("Orig pswd: " + new String(origPswd));
		return compareBytes(hash, origPswd);
				
	}

	private static boolean compareBytes(byte[] pswd, byte[] resPswd) {
		// this ensures each byte is compared before the result is returned
		// in this way the comparison takes the same length of time
		 int diff = pswd.length ^ resPswd.length;
		 for(int i = 0; i < pswd.length && i < resPswd.length; i++)
			 diff |= pswd[i] ^ resPswd[i];
		 log.debug("compareBytes: " + (diff == 0));
		 return diff == 0;
	}
	
	
	private static byte[] hashPswd(byte[] pswd) throws NoSuchAlgorithmException {
		log.debug("pre hash pswd: " + new String(pswd));
		MessageDigest digest = MessageDigest.getInstance("SHA-256");
		byte[] hash = digest.digest(pswd);
		log.debug("post hash pswd: " + new String(hash));
		return hash;
	}
	
	private static int generateSalt() throws NoSuchAlgorithmException {
		// create a secure random number generator
		 SecureRandom sr;
		sr = SecureRandom.getInstance("SHA1PRNG");
		int salt = sr.nextInt(); 
		return salt;
	}
}
