package softwarepulse.util;

import java.awt.AlphaComposite;
import java.awt.Graphics2D;
import java.awt.image.BufferedImage;
import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.IOException;
import java.net.URL;

import javax.imageio.ImageIO;
import org.apache.commons.codec.binary.Base64;

public class ImageHandler {

	public static byte[] encodeImage(String filename) throws IOException {
		ByteArrayOutputStream byteArrayOutputStream;
		byte[] encodedImage;
		
		 if(filename == null) {
			 return null;
		 }
		 else if(filename.isEmpty()) {
			 return null;
		 }
		 else {
			 File fileImage = new File(filename);
			 BufferedImage img = ImageIO.read(fileImage);
			 byteArrayOutputStream = new ByteArrayOutputStream();
			 ImageIO.write(img, "png", byteArrayOutputStream);
			 byteArrayOutputStream.flush();
			 encodedImage = Base64.encodeBase64(byteArrayOutputStream.toByteArray());
			 byteArrayOutputStream.close();
			 return encodedImage; 
		 }
	}
	
	public static byte[] encodeImage(BufferedImage bImage) throws IOException {
		ByteArrayOutputStream byteArrayOutputStream;
		byte[] encodedImage;
		
			 byteArrayOutputStream = new ByteArrayOutputStream();
			 ImageIO.write(bImage, "png", byteArrayOutputStream);
			 byteArrayOutputStream.flush();
			 encodedImage = Base64.encodeBase64(byteArrayOutputStream.toByteArray());
			 byteArrayOutputStream.close();
			 return encodedImage; 
	}
	
	public static BufferedImage decodeImage(byte[] encodedImage) throws IOException {
		ByteArrayInputStream byteArrayInputStream =	new ByteArrayInputStream(Base64.decodeBase64(encodedImage));
		BufferedImage image = ImageIO.read(byteArrayInputStream);
		byteArrayInputStream.close();
		return image;
	}
	
	public static File getFile(String fileLocation) {
		 if(fileLocation == null) {
			 return null;
		 }
		 else if(fileLocation.isEmpty()) {
			 return null;
		 }
		 else {
			 return new File(fileLocation);
		 }
	}
	
	public static BufferedImage getImageFromFile(File file) throws IOException {
		 return ImageIO.read(file);
	}

	/*
	 * Produces a translucent image from a file
	 * passed a filename, file path and a percentage 
	 * returns a BufferedImage  
	 */
	public static BufferedImage getTranslucentPanelImage(String filename, String path, float transparancy) {
		ResourceWrapper iconResourceWrapper = new ResourceWrapper(filename, path);
		URL myurl = ImageHandler.class.getResource(iconResourceWrapper.getAbsoluteName());
		BufferedImage loaded = null;
		try {
			loaded = ImageIO.read(myurl);
		} catch (IOException e1) {
			e1.printStackTrace();
		}
		
        // Create the image using the BufferImage   
        BufferedImage aimg = new BufferedImage(loaded.getWidth(), loaded.getHeight(), BufferedImage.TRANSLUCENT);
//		System.out.println("Width: " + loaded.getWidth());
//		System.out.println("Height: " + loaded.getHeight());
        // Get the images graphics  
        Graphics2D g = aimg.createGraphics();  
        // Set the Graphics composite to Alpha  
        g.setComposite(AlphaComposite.getInstance(AlphaComposite.SRC_OVER, transparancy));  
	    // Draw the LOADED img into the prepared receiver image  
        g.drawImage(loaded, null, 0, 0);  
        // let go of all system resources in this Graphics  
        g.dispose();  
        // Return the image  
        return aimg;  
		
	}

}
